// TwitterUpdater.java
//
// Informatics 45 Spring 2010
// Code Example: Posting Twitter Updates from a Java Program
//
// This program uses Twitter's HTTP-based API to post a Twitter update.
// Notice how much of the code here is the same as the previous program,
// which downloaded a web page and printing its HTML source code to the
// console.  That's because the act of calling an HTTP-based web service
// is, in essence, identical to the act of downloading a web page:
//
// * A URL is used to identify the service to be called and the parameters
//   to be passed to it.
//
// * The result of making a connection and download that URL is whatever
//   the web service returns.  In this case, we just print the result to
//   the console, as we did before.
//
// There are two things that arise in this program that didn't show up in
// the previous one:
//
// * The Twitter API supports "HTTP Basic Authentication" to specify a
//   username and password, which are required in order to post a Twitter
//   update.  This program hooks into built-in Java support for Basic
//   Authentication by creating an "Authenticator."  (Note that Twitter's
//   support for Basic Authentication is being phased out and replaced
//   with something else called OAuth, which is somewhat more cumbersome.
//   For our work this quarter, we'll continue using Basic Authentication,
//   but feel free to look more into OAuth if you'd prefer to write a
//   program that will have a longer shelf life.)
//
// * The Twitter API call to post a Twitter update requires a POST call,
//   with the content of the update being specified as the POST attachment.
//   This program demonstrates writing to the OutputStream associated with
//   a URLConnection, which is how you include POST data.  (Note that we
//   URL encode the POST data, as well, since that is required by the
//   Twitter API.)
//
// Like the previous program, this program is an exploratory example, so
// no attempt was made to organize the code or handle exceptions.

package inf45.spring2010.examples.web2;

import java.io.PrintWriter;
import java.net.Authenticator;
import java.net.PasswordAuthentication;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.Scanner;


public class TwitterUpdater
{
	public static void main(String[] args)
	throws Exception
	{
		// We'll start by asking the user what update to post.  Like the
		// main Twitter web page, we'll ask the question "What are you doing?"
		Scanner consoleIn = new Scanner(System.in);
		
		System.out.print("What are you doing? ");
		String status = consoleIn.nextLine();
		

		// This is something you'd only have to do once in your program,
		// rather than doing it every time you make a Twitter request.  (Or,
		// at least, you'd have to do it once every time you switched from
		// one Twitter user to another, if your program allows multiple
		// users.)
		//
		// The Twitter methods that deal with user-specific commands (like
		// posting an update) require authentication, so that it will be clear
		// who is posting the message and, of course, that the caller has
		// access to post the message on that user's behalf.  Twitter uses
		// HTTP Basic Authentication, which we can set up support for using
		// the following code.  This code says "Whenever the server asks for
		// a username and password, here's what you should return."
		//
		// It's not exactly clear to me why the username can be a string while
		// the password has to be an array of characters.  I'm sure there's
		// an explanation, but I have yet to look it up.  Sometimes it's better
		// just to move forward and get work done.

		Authenticator.setDefault(
			new Authenticator()
			{
				protected PasswordAuthentication getPasswordAuthentication()
				{
					// You'll need to replace this username and password with
					// your own in order for this program to work.
					return new PasswordAuthentication(
						"myusername", "mypassword".toCharArray());
				}
			});
		
		// We connect to Twitter the same way we downloaded a web page:
		// create a URL, then open a URLConnection from it.
		URL updateURL = new URL("http://api.twitter.com/1/statuses/update.xml");
		URLConnection updateConnection = updateURL.openConnection();

		// This calls sets this request up as a POST.  It's a lousy name --
		// setDoOutput? -- but it's saying "We want to include our own data
		// along with this request," which means it's a POST.
		updateConnection.setDoOutput(true);
		
		// Here's how we include our POST data: write to the connection's
		// output stream.  This works just like writing to sockets as we did
		// before.
		PrintWriter connectionOut =
			new PrintWriter(updateConnection.getOutputStream(), true);
		
		// The Twitter API asks us to URL encode our new status, so we're
		// doing that here by calling URLEncoder.encode().  There are
		// different ways to encode the URL; we've chosen one called "UTF-8"
		// (which is an encoding -- associated with the Unicode character set
		// that Java uses -- that, notably, Twitter's API asks us to use).
		connectionOut.println("status=" + URLEncoder.encode(status, "UTF-8"));
		
		// Here, we'll read Twitter's response and just print it to the
		// console.  Notice that we get back XML.  It's not going to be
		// reasonable in your project to just print the XML; instead, you're
		// going to want to use it (or discard it if you don't need it).
		Scanner connectionIn = new Scanner(updateConnection.getInputStream());
		
		while (connectionIn.hasNextLine())
		{
			System.out.println(connectionIn.nextLine());
		}
		
		// Now that we're finished with the connection, we'll close its input
		// stream and output stream.
		connectionIn.close();
		connectionOut.close();
	}
}
